"""Generate simple tables in terminals from a nested list of strings.

Use SingleTable or DoubleTable instead of AsciiTable for box-drawing characters.

https://github.com/Robpol86/terminaltables
https://pypi.python.org/pypi/terminaltables
"""

from terminaltables.tables import AsciiTable, DoubleTable, SingleTable


assert AsciiTable
assert DoubleTable
assert SingleTable
